@@ -1,15 +1,17 @@
 package server
 
 import (
-	"encoding/json"
 	"fmt"
 	"net/url"
 
+	"github.com/clarketm/json"
 	igntypes "github.com/coreos/ignition/config/v2_2/types"
+	"github.com/vincent-petithory/dataurl"
+	"k8s.io/apimachinery/pkg/runtime"
+
 	mcfgv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 	ctrlcommon "github.com/openshift/machine-config-operator/pkg/controller/common"
 	daemonconsts "github.com/openshift/machine-config-operator/pkg/daemon/constants"
-	"github.com/vincent-petithory/dataurl"
 )
 
 const (
@@ -36,39 +38,51 @@ const (
 type kubeconfigFunc func() (kubeconfigData []byte, rootCAData []byte, err error)
 
 // appenderFunc appends Config.
-type appenderFunc func(*mcfgv1.MachineConfig) error
+type appenderFunc func(*igntypes.Config, *mcfgv1.MachineConfig) error
 
 // Server defines the interface that is implemented by different
 // machine config server implementations.
 type Server interface {
-	GetConfig(poolRequest) (*igntypes.Config, error)
+	GetConfig(poolRequest) (*runtime.RawExtension, error)
 }
 
 func getAppenders(currMachineConfig string, f kubeconfigFunc, osimageurl string) []appenderFunc {
 	appenders := []appenderFunc{
 		// append machine annotations file.
-		func(mc *mcfgv1.MachineConfig) error { return appendNodeAnnotations(&mc.Spec.Config, currMachineConfig) },
+		func(cfg *igntypes.Config, mc *mcfgv1.MachineConfig) error {
+			return appendNodeAnnotations(cfg, currMachineConfig)
+		},
 		// append pivot
-		func(mc *mcfgv1.MachineConfig) error { return appendInitialPivot(&mc.Spec.Config, osimageurl) },
+		func(cfg *igntypes.Config, mc *mcfgv1.MachineConfig) error { return appendInitialPivot(cfg, osimageurl) },
 		// append kubeconfig.
-		func(mc *mcfgv1.MachineConfig) error { return appendKubeConfig(&mc.Spec.Config, f) },
+		func(cfg *igntypes.Config, mc *mcfgv1.MachineConfig) error { return appendKubeConfig(cfg, f) },
 		// append the machineconfig content
-		//nolint:gocritic
-		func(mc *mcfgv1.MachineConfig) error { return appendInitialMachineConfig(mc) },
+		appendInitialMachineConfig,
+		// This has to come last!!!
+		appendEncapsulated,
 	}
 	return appenders
 }
 
-// machineConfigToIgnition converts a MachineConfig object into Ignition.
-func machineConfigToIgnition(mccfg *mcfgv1.MachineConfig) *igntypes.Config {
-	tmpcfg := mccfg.DeepCopy()
-	tmpcfg.Spec.Config = ctrlcommon.NewIgnConfig()
+func appendEncapsulated(conf *igntypes.Config, mc *mcfgv1.MachineConfig) error {
+	// since we're encapsulating the MC, we don't need the whole ignition section, that's why
+	// we do this dance here to empty out the ignition section itself. We're just interested into the MC fields
+	tmpIgnCfg := ctrlcommon.NewIgnConfig()
+	rawTmpIgnCfg, err := json.Marshal(tmpIgnCfg)
+	if err != nil {
+		return fmt.Errorf("error marshalling Ignition config: %v", err)
+	}
+	tmpcfg := mc.DeepCopy()
+	tmpcfg.Spec.Config.Raw = rawTmpIgnCfg
 	serialized, err := json.Marshal(tmpcfg)
 	if err != nil {
-		panic(err.Error())
+		return fmt.Errorf("error marshalling MachineConfig: %v", err)
 	}
-	appendFileToIgnition(&mccfg.Spec.Config, daemonconsts.MachineConfigEncapsulatedPath, string(serialized))
-	return &mccfg.Spec.Config
+	err = appendFileToIgnition(conf, daemonconsts.MachineConfigEncapsulatedPath, string(serialized))
+	if err != nil {
+		return fmt.Errorf("error appending file to raw Ignition config: %v", err)
+	}
+	return nil
 }
 
 // Golang :cry:
@@ -82,7 +96,11 @@ func appendInitialPivot(conf *igntypes.Config, osimageurl string) error {
 	}
 
 	// Tell pivot.service to pivot early
-	appendFileToIgnition(conf, daemonconsts.EtcPivotFile, osimageurl+"\n")
+	err := appendFileToIgnition(conf, daemonconsts.EtcPivotFile, osimageurl+"\n")
+	if err != nil {
+		return err
+	}
+
 	// Awful hack to create a file in /run
 	// https://github.com/openshift/machine-config-operator/pull/363#issuecomment-463397373
 	// "So one gotcha here is that Ignition will actually write `/run/pivot/image-pullspec` to the filesystem rather than the `/run` tmpfs"
@@ -108,12 +126,12 @@ WantedBy=multi-user.target
 // by the MCS when the node first booted.  This currently is only used as a debugging aid
 // in cases where there is unexpected "drift" between the initial bootstrap MC/Ignition and the one
 // computed by the cluster.
-func appendInitialMachineConfig(mc *mcfgv1.MachineConfig) error {
+func appendInitialMachineConfig(conf *igntypes.Config, mc *mcfgv1.MachineConfig) error {
 	mcJSON, err := json.MarshalIndent(mc, "", "    ")
 	if err != nil {
 		return err
 	}
-	appendFileToIgnition(&mc.Spec.Config, machineConfigContentPath, string(mcJSON))
+	appendFileToIgnition(conf, machineConfigContentPath, string(mcJSON))
 	return nil
 }
 
@@ -122,7 +140,10 @@ func appendKubeConfig(conf *igntypes.Config, f kubeconfigFunc) error {
 	if err != nil {
 		return err
 	}
-	appendFileToIgnition(conf, defaultMachineKubeConfPath, string(kcData))
+	err = appendFileToIgnition(conf, defaultMachineKubeConfPath, string(kcData))
+	if err != nil {
+		return err
+	}
 	return nil
 }
 
@@ -131,7 +152,10 @@ func appendNodeAnnotations(conf *igntypes.Config, currConf string) error {
 	if err != nil {
 		return err
 	}
-	appendFileToIgnition(conf, daemonconsts.InitialNodeAnnotationsFilePath, anno)
+	err = appendFileToIgnition(conf, daemonconsts.InitialNodeAnnotationsFilePath, anno)
+	if err != nil {
+		return err
+	}
 	return nil
 }
 
@@ -148,7 +172,7 @@ func getNodeAnnotation(conf string) (string, error) {
 	return string(contents), nil
 }
 
-func appendFileToIgnition(conf *igntypes.Config, outPath, contents string) {
+func appendFileToIgnition(conf *igntypes.Config, outPath, contents string) error {
 	fileMode := int(420)
 	file := igntypes.File{
 		Node: igntypes.Node{
@@ -166,6 +190,7 @@ func appendFileToIgnition(conf *igntypes.Config, outPath, contents string) {
 		conf.Storage.Files = make([]igntypes.File, 0)
 	}
 	conf.Storage.Files = append(conf.Storage.Files, file)
+	return nil
 }
 
 func getDecodedContent(inp string) (string, error) {
